#include "stdafx.h"
#include <stdio.h>
#include <time.h>        // PLATFORM_SPECIFIC for time routines
#include <sys/timeb.h>   // PLATFORM_SPECIFIC for time routines

#include "toolkit.h"

Timer::Timer(void)
{
   this->ivName  = NULL;
   this->ivSecs  = 0L;
   this->ivMSecs = 0L;
}

Timer::Timer(char *pName)
{
   this->ivName  = pName;
   this->ivSecs  = 0L;
   this->ivMSecs = 0L;
}

Timer::~Timer(void)
{
   if (this->isRunning() == true)
   {
      this->stopTimer();
   }
}

bool Timer::isRunning(void)
{
   return (this->ivSecs != 0L || this->ivMSecs != 0L);
}


// PLATFORM_SPECIFIC
// this routine for getting the current time in milliseconds since a fixed
// point in the past is platform-specific, and thus has been clearly set aside
// as such. It is private, and so can only be used by Time itself.
// When porting MINTEL to another system, this almost certainly will have to
// be adapted! The fixed reference for the Windows platform is:
//
// midnight (00:00:00), January 1, 1970, coordinated universal time (UTC).
//
// Since MINTEL is supposed to run on a single machine for now, and times are
// only used relatively, the timezone info is discarded, and the above reference
// point is non-relevant.
//
// Parameter       Type       I/O  Explanation
// ==============  =========  ===  =============================================
// return value    void            multiple returned values through parameters. 
// pSeconds        long *     OUT  seconds of current time, see above. 
// pMilliseconds   long *     OUT  milliseconds of current time, see above. 
//

void Timer::getCurrentTime(long *pSeconds, long *pMilliseconds)
{
   // define the struct needed for _ftime
   struct __timeb64 lTimeStruct;

   // retrieve time in the struct
   _ftime64_s(&lTimeStruct);

   // transport the relevant fields to the output parameters
   *pSeconds      = (long)(lTimeStruct.time);
   *pMilliseconds = (long)(lTimeStruct.millitm);
}

// PLATFORM_SPECIFIC
// This method builds a sortable timestamp string, based on current time
//
// Parameter       Type       I/O  Explanation
// ==============  =========  ===  =============================================
// return value    char *     OUT  the pointer to the timestamp string. 
//

char *Timer::getTimeStamp(void)
{
   time_t           lTime;
   struct tm        lToday;
   struct __timeb64 lTimeStruct;
   static char      lTimeStamp[MAX_STRINGBUFFER];

   // Use time structure to build a customized time string.
   time(&lTime);
   localtime_s( &lToday, &lTime );
   strftime(lTimeStamp, MAX_STRING, "%Y-%m-%d %H:%M:%S", &lToday);

   // retrieve time in the struct.
   _ftime64_s(&lTimeStruct);

   // transport the relevant fields to the output parameters.
   sprintf_s(lTimeStamp, MAX_STRING, "%s.%03ld", lTimeStamp, lTimeStruct.millitm);

   // return the complete string accurate to milliseconds. 
   return lTimeStamp;
}

void Timer::startTimer(void)
{
   // load current time into structure
   this->getCurrentTime(&(this->ivSecs), &(this->ivMSecs));

   // write the starting message
   Logger::Log(MT_TIMING, _FILE_, __LINE__, RC_OK, "Timer[%08X] started", this);
}

void Timer::startTimer(char *pName)
{
   this->ivName = pName;

   // load current time into structure
   this->getCurrentTime(&(this->ivSecs), &(this->ivMSecs));

   // write the starting message
   Logger::Log(MT_TIMING, _FILE_, __LINE__, RC_OK, "Timer[%08X] started", this);
}

void Timer::stopTimer(void)
{
   long lSecs = 0;
   long lMsecs = 0;

   // get current time
   this->getCurrentTime(&lSecs, &lMsecs);
   
   // calculate difference
   long lDiffSecs = lSecs - this->ivSecs;
   long lDiffMsecs = lMsecs - this->ivMSecs;
   if (lDiffMsecs < 0)
   {
      lDiffMsecs += 1000;
      lDiffSecs--;
   }

   // reset the timer
   this->ivSecs = 0L;
   this->ivMSecs = 0L;

   // write the total message
   if (this->ivName == NULL)
   {
      Logger::Log(MT_TIMING, _FILE_, __LINE__, RC_OK, 
                  "Timer[%08X]: %ld.%03ld seconds", 
                  this, lDiffSecs, lDiffMsecs); 
   }
   else
   {
      Logger::Log(MT_TIMING, _FILE_, __LINE__, RC_OK, 
                  "Timer[%08X]: %ld.%03ld seconds (%s)", 
                  this, lDiffSecs, lDiffMsecs, this->ivName); 
   }
}

#ifdef TOOLKIT_SELFTEST
RetCode Timer::selfTest(void)
{
   RetCode lReturn = RC_OK;

   // Note the start of the selfTest in the logfile
   Logger::doubleLine();
   Logger::Log(MT_INFO, _FILE_, __LINE__, RC_OK, "Starting selfTest for Timer class");

   long lCounter;

   TK_TIMER_DEF(lTimer1, "timer defined through macros");
   TK_TIMER_START(lTimer1);
   for (lCounter = 0; lCounter < 200000000; lCounter++);
   TK_TIMER_STOP(lTimer1);

   Timer lTimer2("timer defined through code");
   lTimer2.startTimer();
   for (lCounter = 0; lCounter < 200000000; lCounter++);
   lTimer2.stopTimer();

   Timer *lTimer3 = new Timer("timer defined through allocation");
   lTimer3->startTimer();
   for (lCounter = 0; lCounter < 200000000; lCounter++);
   delete lTimer3;

   // Note the end of the selfTest in the logfile
   Logger::Log(MT_INFO, _FILE_, __LINE__, RC_OK, "End of selfTest for Timer class"); 
   Logger::doubleLine();

   return lReturn;
}
#endif
